@@ -1,10 +1,10 @@
 import argparse
+import dataclasses
 import functools
 import os
 import pathlib
 import re
 import sys
-import textwrap
 from types import ModuleType
 from typing import (
     Any,
@@ -29,12 +29,22 @@ from gitlab.base import RESTObject
 camel_upperlower_regex = re.compile(r"([A-Z]+)([A-Z][a-z])")
 camel_lowerupper_regex = re.compile(r"([a-z\d])([A-Z])")
 
+
+@dataclasses.dataclass
+class CustomAction:
+    required: Tuple[str, ...]
+    optional: Tuple[str, ...]
+    in_object: bool
+    requires_id: bool  # if the `_id_attr` value should be a required argument
+    help: Optional[str]  # help text for the custom action
+
+
 # custom_actions = {
 #    cls: {
-#        action: (mandatory_args, optional_args, in_obj),
+#        action: CustomAction,
 #    },
 # }
-custom_actions: Dict[str, Dict[str, Tuple[Tuple[str, ...], Tuple[str, ...], bool]]] = {}
+custom_actions: Dict[str, Dict[str, CustomAction]] = {}
 
 
 # For an explanation of how these type-hints work see:
@@ -44,38 +54,14 @@ custom_actions: Dict[str, Dict[str, Tuple[Tuple[str, ...], Tuple[str, ...], bool
 __F = TypeVar("__F", bound=Callable[..., Any])
 
 
-class VerticalHelpFormatter(argparse.HelpFormatter):
-    def format_help(self) -> str:
-        result = super().format_help()
-        output = ""
-        indent = self._indent_increment * " "
-        for line in result.splitlines(keepends=True):
-            # All of our resources are on one line and wrapped inside braces.
-            # For example: {application,resource1,resource2}
-            # except if there are fewer resources - then the line and help text
-            # are collapsed on the same line.
-            # For example:  {list}      Action to execute on the GitLab resource.
-            # We then put each resource on its own line to make it easier to read.
-            if line.strip().startswith("{"):
-                choice_string, help_string = line.split("}", 1)
-                choice_list = choice_string.strip(" {").split(",")
-                help_string = help_string.strip()
-
-                if help_string:
-                    help_indent = len(max(choice_list, key=len)) * " "
-                    choice_list.append(f"{help_indent} {help_string}")
-
-                choices = "\n".join(choice_list)
-                line = f"{textwrap.indent(choices, indent)}\n"
-            output += line
-        return output
-
-
 def register_custom_action(
+    *,
     cls_names: Union[str, Tuple[str, ...]],
-    mandatory: Tuple[str, ...] = (),
+    required: Tuple[str, ...] = (),
     optional: Tuple[str, ...] = (),
     custom_action: Optional[str] = None,
+    requires_id: bool = True,  # if the `_id_attr` value should be a required argument
+    help: Optional[str] = None,  # help text for the action
 ) -> Callable[[__F], __F]:
     def wrap(f: __F) -> __F:
         @functools.wraps(f)
@@ -98,7 +84,13 @@ def register_custom_action(
                 custom_actions[final_name] = {}
 
             action = custom_action or f.__name__.replace("_", "-")
-            custom_actions[final_name][action] = (mandatory, optional, in_obj)
+            custom_actions[final_name][action] = CustomAction(
+                required=required,
+                optional=optional,
+                in_object=in_obj,
+                requires_id=requires_id,
+                help=help,
+            )
 
         return cast(__F, wrapped_f)
 
@@ -134,7 +126,6 @@ def _get_base_parser(add_help: bool = True) -> argparse.ArgumentParser:
     parser = argparse.ArgumentParser(
         add_help=add_help,
         description="GitLab API Command Line Interface",
-        formatter_class=VerticalHelpFormatter,
         allow_abbrev=False,
     )
     parser.add_argument("--version", help="Display the version.", action="store_true")
@@ -292,6 +283,12 @@ def _get_base_parser(add_help: bool = True) -> argparse.ArgumentParser:
         action="store_true",
         default=os.getenv("GITLAB_SKIP_LOGIN"),
     )
+    parser.add_argument(
+        "--no-mask-credentials",
+        help="Don't mask credentials in debug mode",
+        dest="mask_credentials",
+        action="store_false",
+    )
     return parser
 
 
@@ -379,29 +376,31 @@ def main() -> None:
     gitlab_resource = args.gitlab_resource
     resource_action = args.resource_action
     skip_login = args.skip_login
+    mask_credentials = args.mask_credentials
 
     args_dict = vars(args)
     # Remove CLI behavior-related args
     for item in (
-        "gitlab",
+        "api_version",
         "config_file",
-        "verbose",
         "debug",
-        "gitlab_resource",
-        "resource_action",
-        "version",
-        "output",
         "fields",
+        "gitlab",
+        "gitlab_resource",
+        "job_token",
+        "mask_credentials",
+        "oauth_token",
+        "output",
+        "pagination",
+        "private_token",
+        "resource_action",
         "server_url",
+        "skip_login",
         "ssl_verify",
         "timeout",
-        "api_version",
-        "pagination",
         "user_agent",
-        "private_token",
-        "oauth_token",
-        "job_token",
-        "skip_login",
+        "verbose",
+        "version",
     ):
         args_dict.pop(item)
     args_dict = {k: _parse_value(v) for k, v in args_dict.items() if v is not None}
@@ -409,7 +408,7 @@ def main() -> None:
     try:
         gl = gitlab.Gitlab.merge_config(vars(options), gitlab_id, config_files)
         if debug:
-            gl.enable_debug()
+            gl.enable_debug(mask_credentials=mask_credentials)
         if not skip_login and (gl.private_token or gl.oauth_token):
             gl.auth()
     except Exception as e:
